var api = require("../../utils/api.js");
var payUtils = require("../../utils/payUtils.js");
import { ACTION_GO_SCAN, ACTION_GO_HOME } from '../../const/actioncmd'
import { ACTION_ON_PAY_SUCCESS } from '../../const/actioncmd'
import { ACTION_ON_PAY_FAILED } from '../../const/actioncmd'
import { ACTION_BALANCE_BACK_GOODS } from '../../const/actioncmd'

Page({
  data: {
    goodsNum: 0,
    payPrice: 0.00,
    totalPrice: 0.00,
    couponPrice: 0.00,
    qrPayShow: false,
    list: [],
    orderNo: "",
    isPaying: false,
  },

  //回退到上一个页面
  events: {
    onBack(e) {
      console.log('onBack触发');
      my.ix.sendBuddyMessage({
        // 填入目标小程序的 AppID
        target: getApp().globalData.backScreenAppId,
        data: {
          action: ACTION_BALANCE_BACK_GOODS,
          data: null
        },
        success: (res) => {
          e.preventDefault();
        },
        fail: (res) => {
          console.info(`sendBuddyMessage failed: ${JSON.stringify(res)}`);
        }
      });
    }
  },


  onReady() {
    if (my.canIUse('hideBackHome')) {
      my.hideBackHome();
    }
  },

  onLoad(options) {
    var totalQuantity = options.totalQuantity
    var totalPrice = options.totalPrice
    var totalDisc = options.totalDisc
    this.setData({
      list: options.goods,
    });

    if (totalQuantity != null) {
      this.setData({
        goodsNum: totalQuantity,
      })
    }

    if (totalPrice != null) {
      this.setData({
        payPrice: totalPrice - totalDisc,
        totalPrice: totalPrice,
        couponPrice: totalDisc,
      })
    }

    //商品数据
    let context = this;
    my.getStorage({
      key: 'goods',
      success: function (res) {
        console.log("商品信息 ===》  " + JSON.stringify(res));
        context.setData({
          list: res.data
        })
      },
      fail: function (res) {
        console.log("商品信息 ===》  " + JSON.stringify(res));
      }
    });

    var speechContent = this.data.payPrice + "元";
    my.ix.speech({
      text: speechContent,
      speak: true,
      success: (r) => {
        my.ix.speech({
          text: '请刷脸或扫码支付',
          speak: true,
          success: (r) => {
          }
        })
      }
    });

  },

  onReady() {
    if (my.canIUse('hideBackHome')) {
      my.hideBackHome();
    }
  },


  onShow() {
    my.ix.onBuddyMessage({
      success: (res) => {
        console.info(`onBuddyMessage success: ${JSON.stringify(res)}`);
        if (res.action == ACTION_GO_SCAN) {//返回扫码页
          //增加跳转回去的内容
          my.navigateBack();
        } else if (res.action == ACTION_GO_HOME) {
          //去首页
          my.reLaunch({
            url: "/pages/home/home",
          });
        }
      }
    });
  },

  onHide() {
    // 页面隐藏
    //关闭后屏监听
    my.ix.offBuddyMessage();
  },

  /**
 * @todo 支付结果的获取
 * @todo 跳转数据的保存
 */
  startPay() {
    //开始loading
    my.showLoading({});

    //获取order
    var payAmount = payUtils.yuan2Fen(this.data.payPrice);
    payUtils.getOrder(/*payAmount*/1, this.data.list, function (res) {
      if (res != null) {

        console.log("callback调用,res=" + JSON.stringify(res));
        if (res.resultCode == 1) {
          this.setData({
            orderNo: res.data
          });
          this.payWithOpenCashier(res.data);
        }
      }
    }.bind(this));
  },

  /**
   * 打开收银台，进行扫码收款
   */
  payWithOpenCashier() {
    try {
      console.info("收银台版本" + my.ix.getVersionSync({
        packageName: 'zoloz.phone.android.alipay.com.dragonfly'
      }).versionName);

      //开启收银台监听
      my.ix.startApp({
        appName: "cashier",
        bizNo: this.data.orderNo,
        totalAmount: this.data.payPrice,
        success: (res) => {
          console.log("收银台启动回调 success==>" + JSON.stringify(res));
          if (res.codeType == "F") {
            this.doPay(this.data.orderNo, res.barCode);
          } else {
            this.doPay(this.data.orderNo, res.barCode);
          }
        }, fail: (res) => {
          my.hideLoading();
          my.showToast({
            type: 'exception',
            content: api.decodeErrorMessage(res.error),
            duration: 2000
          });
        }
      });
      //收银台回调
      my.ix.onCashierEventReceive((res) => {
        console.log("onCashierEventReceive==>" + JSON.stringify(res));

        if (res.bizType == 'RESULT_CLOSED' || res.status == "RESULT_CLOSE") {
          console.log("收银台关闭");
          //关闭监听
          my.ix.offCashierEventReceive();
          if (this.data.isPaying == true) {
            //如果正在支付中，则等待支付结果返回
          } else {
            //跳转支付结果页面，说明支付已完成,无论取消支付还是支付成功等都跳转支付结果页

            // if (this.data.actualPayPriceYuan != null) {
            //   my.redirectTo({
            //     //payPrice和couponPrice需要考虑支付宝优惠
            //     url: '/pages/paysuccess/paysuccess?orderPrice=' + this.data.totalPrice + "&couponPrice=" + this.data.couponPrice + "&payPrice=" + this.data.payPrice+"&totalCount="+this.data.totalCount,
            //   });
            // } else {
            //   //说明用户取消了支付
            //   my.redirectTo({
            //     url: '/pages/paysuccess/paysuccess?payFailedMsg=' + "用户取消" + "&payPriceYuan=" + this.data.payPriceYuan,
            //   });
            // }
          }
        } else if (res.bizType == 'RESULT_BTN_FUNCTION')
          console.log("收银台自定义按钮按下");
        else if (res.bizType == 'RESULT_MEMBER')
          console.log("支付结果页会员开卡");
        else
          console.log('RESULT: ' + res.bizType);
      });
    } catch (err) {
      console.log("收银台启动回调 异常==>" + JSON.stringify(err));
    }

  },


  //调用收款接口
  doPay(orderNo, payCode) {
    console.log("doPay调用");
    this.setData({ isPaying: true });
    //使用barCode来当面付
    var payAmount = payUtils.yuan2Fen(this.data.payPrice);
    payUtils.startPay(orderNo, payCode, /*payAmount*/1, this.data.list, function (res) {
      my.hideLoading();
      console.log("支付回调  ===>" + JSON.stringify(res));
      this.setData({ isPaying: false });
      if (res == null) {
        return;
      }
      if (res.resultCode == 1) {
        //成功
        console.log("支付回调成功，跳转 ");

        if (res.data.tradeAmount != null) {
          var speechContent = "成功支付" + payUtils.fen2Yuan(res.data.tradeAmount) + "元";
          my.ix.speech({
            text: speechContent,
            speak: true,
            success: (r) => {
            }
          });
        }

        my.ix.offCashierEventReceive();
        this.saveData(this.data.list, res.data);
        my.redirectTo({
          url: '/pages/paysuccess/paysuccess',
        });

        //发送支付成功到后屏
        my.ix.sendBuddyMessage({
          // 填入目标小程序的 AppID
          target: getApp().globalData.backScreenAppId,
          data: {
            action: ACTION_ON_PAY_SUCCESS,
            data: res.data
          },
          success: (res) => {
            console.info(`sendBuddyMessage success: ${JSON.stringify(res)}`);
          },
          fail: (res) => {
            console.info(`sendBuddyMessage failed: ${JSON.stringify(res)}`);
          }
        });
      } else {
        console.log("支付回调失败，toast提示  ");
        my.ix.offCashierEventReceive();
        my.showToast({
          type: 'fail',
          content: res.message,
          duration: 2000
        });
        //发送支付失败到后屏
        my.ix.sendBuddyMessage({
          // 填入目标小程序的 AppID
          target: getApp().globalData.backScreenAppId,
          data: {
            action: ACTION_ON_PAY_FAILED,
            data: null
          },
          success: (res) => {
            console.info(`sendBuddyMessage success: ${JSON.stringify(res)}`);
          },
          fail: (res) => {
            console.info(`sendBuddyMessage failed: ${JSON.stringify(res)}`);
          }
        });
      }
    }.bind(this));
  },


  /**
   * 保存最近支付成功的相关信息，每次支付成功都需刷新
   * @param paySuccessGoodsList 支付成功的商品列表信息
   * @param paySuccessResponse 支付成功响应,即云中台返回响应中的data信息
   */
  saveData(paySuccessGoodsList, paySuccessResponse) {
    my.setStorageSync({
      key: 'paySuccessGoodsList',
      data: paySuccessGoodsList
    });
    my.setStorageSync({
      key: 'paySuccessResponse',
      data: paySuccessResponse
    });
  },


});
